@@ -22,6 +22,10 @@
 #' you can provide a list with the names \code{'continuous'}, \code{'proportion'}, \code{'factor'} and
 #' the function will choose accordingly. If you fail to define a certain category
 #' it will default to the above.
+#' 
+#' You can also use a custom function that returns a string with the attribute 'colname' set that will be appended
+#' to the results instead of the p-value column.
+#' to the results instead of the p-value column.
 #'
 #' @param x The variable that you want the statistics for
 #' @param by The variable that you want to split into different
@@ -54,6 +58,7 @@
 #'  to limit publication associated issues.
 #' @param statistics.sig_lim The significance limit for < sign, i.e. p-value 0.0000312
 #'  should be < 0.0001 with the default setting.
+#' @param statistics.suppress_warnings Hide warnings from the statistics function.
 #' @param show_all_values This is by default false as for instance if there is
 #'  no missing and there is only one variable then it is most sane to only show
 #'  one option as the other one will just be a complement to the first. For instance
@@ -106,6 +111,7 @@ getDescriptionStatsBy <- function(x,
                                   statistics=FALSE,
                                   statistics.sig_lim=10^-4,
                                   statistics.two_dec_lim= 10^-2,
+                                  statistics.suppress_warnings=TRUE,
                                   useNA = c("ifany", "no", "always"),
                                   useNA.digits = digits,
                                   continuous_fn = describeMean,
@@ -148,50 +154,61 @@ getDescriptionStatsBy <- function(x,
 
   useNA <- match.arg(useNA)
 
-  if (is.list(statistics) ||
-        (statistics != FALSE &&
-           !is.function(statistics))){
-    if (is.list(statistics)){
-      types <- c("continuous",
-                 "proportion",
-                 "factor")
-      if (any(!names(statistics) %in% types))
-        stop("If you want to provide custom functions for generating statistics",
-             " you must either provide a function or a list with the elements:",
-             " '", paste(types, collapse="', '"), "'")
-
-      if (is.numeric(x) &&
-            length(unique(x)) != 2){
-        statistics <- statistics[["continuous"]]
-      }else if (length(unique(x)) == 2){
-        if ("proportion" %in% names(statistics)){
-          statistics <- statistics[["proportion"]]
-        }else{
-          statistics <- statistics[["factor"]]
+  if (!is.function(statistics)){
+    if (is.list(statistics) ||
+        (statistics != FALSE)){
+        if (is.list(statistics)){
+          types <- c("continuous",
+                     "proportion",
+                     "factor")
+          if (any(!names(statistics) %in% types))
+            stop("If you want to provide custom functions for generating statistics",
+                 " you must either provide a function or a list with the elements:",
+                 " '", paste(types, collapse="', '"), "'")
+          
+          if (is.numeric(x) &&
+              length(unique(x)) != 2){
+            statistics <- statistics[["continuous"]]
+          }else if (length(unique(x)) == 2){
+            if ("proportion" %in% names(statistics)){
+              statistics <- statistics[["proportion"]]
+            }else{
+              statistics <- statistics[["factor"]]
+            }
+          }else{
+            statistics <- statistics[["factor"]]
+          }
+          
+          if (is.character(statistics))
+            statistics <- get(statistics)
+        }
+        
+        if (!is.function(statistics)){
+          if(length(unique(x)) == 2){
+            statistics <- getPvalFisher
+          }else if(is.numeric(x)){
+            if (length(unique(by)) == 2)
+              statistics <- getPvalWilcox
+            else
+              statistics <- getPvalAnova
+          }else{
+            statistics <- getPvalFisher
+          }
         }
-      }else{
-        statistics <- statistics[["factor"]]
       }
-
-      if (is.character(statistics))
-        statistics <- get(statistics)
+  }
+  
+  if (is.function(statistics)){
+    if (statistics.suppress_warnings){
+      pval <- suppressWarnings(
+        statistics(x = x,
+                   by = by)
+      )
+    }else{
+      pval <- 
+        statistics(x = x,
+                   by = by)
     }
-
-    if (!is.function(statistics)){
-      if(length(unique(x)) == 2){
-        statistics <- getPvalFisher
-      }else if(is.numeric(x)){
-        if (length(unique(by)) == 2)
-          statistics <- getPvalWilcox
-        else
-          statistics <- getPvalAnova
-      }else{
-        statistics <- getPvalFisher
-      }
-    }
-
-    pval <- statistics(x = x,
-                       by = by)
   }
 
   # Always have a total column if the description statistics
@@ -464,38 +481,43 @@ getDescriptionStatsBy <- function(x,
     }
   }
 
-  if (use_units){
-    if(is.logical(use_units)){
-      if (units(x) != ""){
-        unitcol <- rep(sprintf("%s",units(x)), times=NROW(results))
-        unitcol[rownames(results) == "Missing"] <- ""
-      }else{
-        unitcol <- rep("", times=NROW(results))
-      }
-      if (length(unitcol) != nrow(results)){
-        stop("There is an discrepancy in the number of rows in the units",
-             " and the by results: ", length(unitcol), " units vs ", nrow(results), " results",
-             "\n Units:", paste(unitcol, collapse=", "),
-             "\n Rows results:", paste(rownames(results), collapse=", "))
-      }
-      results <- cbind(results, unitcol)
-      cn <- c(cn, "units")
-    }else if(use_units == "name"){
-      if (units(x) != ""){
-        name <- sprintf("%s (%s)", name, units(x))
-      }
+  if (isTRUE(use_units)){
+    if (units(x) != ""){
+      unitcol <- rep(sprintf("%s",units(x)), times=NROW(results))
+      unitcol[rownames(results) == "Missing"] <- ""
     }else{
-      stop("The units use suggested has not yet been implemented, only logical or 'name' are.")
+      unitcol <- rep("", times=NROW(results))
+    }
+    if (length(unitcol) != nrow(results)){
+      stop("There is an discrepancy in the number of rows in the units",
+           " and the by results: ", length(unitcol), " units vs ", nrow(results), " results",
+           "\n Units:", paste(unitcol, collapse=", "),
+           "\n Rows results:", paste(rownames(results), collapse=", "))
+    }
+    results <- cbind(results, unitcol)
+    cn <- c(cn, "units")
+  }else if(use_units == "name"){
+    if (units(x) != ""){
+      name <- sprintf("%s (%s)", name, units(x))
     }
   }
 
   if (is.function(statistics)){
-    pval <- txtPval(pval,
-                    lim.sig =statistics.sig_lim,
-                    lim.2dec = statistics.two_dec_lim,
-                    html = html)
-    results <- cbind(results, c(pval, rep("", nrow(results)-1)))
-    cn <- c(cn, "P-value")
+    if (is.numeric(pval) &&
+        pval <= 1 &&
+        pval >= 0){
+      pval <- txtPval(pval,
+                      lim.sig =statistics.sig_lim,
+                      lim.2dec = statistics.two_dec_lim,
+                      html = html)
+      results <- cbind(results, c(pval, rep("", nrow(results)-1)))
+      cn <- c(cn, "P-value")
+    }else if(is.character(pval) && !is.null(attr(pval, 'colname'))){
+      results <- cbind(results, c(pval, rep("", nrow(results)-1)))
+      cn <- c(cn, attr(pval, 'colname'))
+    }else{
+      stop("Your statistics function should either return a numerical value from 0 to 1 or a character with the attribute 'colname'")
+    }
   }
 
   colnames(results) <- cn
